#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/11/1
 """
from __future__ import unicode_literals, absolute_import
from django.db import models


class PayResult(models.Model):
    """
    支付结果信息
    """
    notify_time = models.DateTimeField('通知时间', auto_now_add=True)
    notify_type = models.CharField(max_length=128, verbose_name='通知类型', null=True)
    notify_id = models.IntegerField(verbose_name='通知id')
    order_id = models.IntegerField('订单id')
    total_fee = models.DecimalField(max_digits=11, decimal_places=2, verbose_name='订单总额', default=0)
    cash_fee = models.DecimalField(max_digits=11, decimal_places=2, verbose_name='实际支付金额', default=0)
    coupon_fee = models.DecimalField(max_digits=11, decimal_places=2, verbose_name='代金券或者优惠金额', default=0)
    coupon_count = models.IntegerField('代金券或立减优惠使用数量', default=0)
    pay_time = models.DateTimeField('支付时间', auto_now_add=True)
    pay_way = models.IntegerField('支付类型', null=True)
    app_id = models.CharField(max_length=64, verbose_name='app id', null=True)
    pay_type = models.CharField(max_length=64, verbose_name='支付类型', null=True, blank=True)
    create_time = models.DateTimeField('记录创建时间', auto_now_add=True)


class AllocateRecord(models.Model):
    """
    会员宝分账记录
    """
    request_no = models.CharField(max_length=128, verbose_name='请求id')
    notify_id = models.IntegerField(verbose_name='通知id')
    app_id = models.CharField(max_length=128, verbose_name='appid 对应 merchantId')
    operator_id = models.PositiveSmallIntegerField('操作id')
    user_id = models.IntegerField('用户id，对应userIn')
    amount = models.DecimalField(max_digits=11, decimal_places=2, verbose_name='金额')
    desc = models.TextField('描述')
    create_time = models.DateTimeField('记录创建时间', auto_now_add=True)


class RefundRecord(models.Model):
    """
    会员宝退款记录
    """
    notify_id = models.IntegerField(verbose_name='通知id')
    amount = models.DecimalField(max_digits=11, decimal_places=2, verbose_name='计划退回金额')
    confirm_amount = models.DecimalField(max_digits=11, decimal_places=2, verbose_name='实际退回金额')
    order_id = models.IntegerField('订单id')
    refund_id = models.CharField(max_length=64,verbose_name='退款id，由会员宝返回')
    user_id = models.IntegerField('退款用户id，即common api的用户id')
    create_time = models.DateTimeField('记录创建时间', auto_now_add=True)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/8/6
 """
from __future__ import unicode_literals, absolute_import
from django.contrib.auth import authenticate
from djdg_core import settings
from django.http.response import JsonResponse
from rest_framework.request import Request
from djdg_core import exceptions
from djdg_core.middleware import ResponseMiddleware
from djdg_oauth.oauthclient import OAuthClient
from djdg_oauth.oauthcore import get_verify_key
from djdg_oauth.oauthcore import getSign
import logging
import json


class AuthenticationMiddleware(object):
    """
    用户认证中间件
    """
    PERMIT_ALL_PARAM = '_permit_all_'

    def __init__(self):
        self.setting = settings.MIDDLEWARE_CONFIG.get(self.__class__.__name__, {})
        self.logger = logging.getLogger(self.setting['LOGGER'])
        self.permit_all_param = self.PERMIT_ALL_PARAM
        self.auth_header_name = self.setting.get('AUTH_HEADER_NAME', 'TOKEN')
        self.auth_token_format = self.setting.get('AUTH_TOKEN_FORMAT', 'session:{token}')
        self.res_mdl_setting = settings.MIDDLEWARE_CONFIG.get(ResponseMiddleware.__name__, {})
        self.status_param = self.res_mdl_setting['STATUS_PARAM']
        self.msg_param = self.res_mdl_setting['MSG_PARAM']
        self.data_param = self.res_mdl_setting['DATA_PARAM']
        self.cors_auth_disable = self.setting.get('CORS_AUTH_DISABLE')

    def process_view(self, request, callback, callback_args, callback_kwargs):
        if self.cors_auth_disable and 'options' == request.method.lower():
            # 跨域时，前端不能往OPTIONS请求中增加TOKEN头，故而此处先忽略
            return None
        if isinstance(request, Request):
            orig_request = request._request
        else:
            orig_request = request
        auth_client = OAuthClient()
        uri, http_method, body, headers = auth_client._extract_params(request)
        signature = headers.get("Authorization")
        if signature:
            ret = self.open_auth(request, body, headers)
        else:
            ret = self.token_auth(orig_request, request)
        # authorization和token均验证不通过，则查看是否是视图允许一切访问
        return None if ret and getattr(callback, self.permit_all_param, None) else ret

    def open_auth(self, request, body, headers):
        err_msg = None
        if isinstance(body, basestring):
            body_obj = json.loads(body)
        else:
            body_obj = body
        appid = body_obj.get("appid")
        keys_obj = None
        if not appid:
            self.logger.error('can not find appid in request body')
            err_msg = '没有找到appid，认证失败！'
        else:
            keys_obj = get_verify_key(appid)
            if not keys_obj:
                self.logger.error('Can not found App [{aid}] secret'.format(aid=appid))
                err_msg = '找不到APP[{aid}]的秘钥！'.format(aid=appid)
        if err_msg or not keys_obj:
            return JsonResponse(
                {
                    self.status_param: exceptions.auth_error.status,
                    self.msg_param: err_msg,
                    self.data_param: exceptions.auth_error.data
                },
                status=401
            )
        signature = headers.get("Authorization")
        request.client_type = keys_obj.app
        user = authenticate(body=body, appid=appid, secret=keys_obj.secret, signature=signature)
        if not user:
            _signature = getSign(body, keys_obj.secret)
            msg = 'App {aid} signature value error [{s1}/{s2}]'.format(
                aid=appid,
                s1=signature,
                s2=_signature
            )
            self.logger.error(msg)
            return JsonResponse(
                {
                    self.status_param: exceptions.auth_error.status,
                    self.msg_param: exceptions.auth_error.msg,
                    self.data_param: exceptions.auth_error.data
                },
                status=401
            )
        request.user = user
        msg = 'App {aid} login with signature {s}'.format(
            aid=user.app_id,
            s=signature
        )
        self.logger.info(msg)

    def token_auth(self, orig_request, request):
        header_token = request.META.get('HTTP_{0}'.format(self.auth_header_name))
        if not header_token:
            self.logger.error('request {rid} has no token'.format(rid=id(request)))
            return JsonResponse(
                {
                    self.status_param: exceptions.auth_error.status,
                    self.msg_param: exceptions.auth_error.msg,
                    self.data_param: exceptions.auth_error.data
                },
                status=401
            )
        only_token = header_token.split(':')[-1]
        auth_token = self.auth_token_format.format(token=only_token)
        try:
            user = authenticate(token=auth_token)
        except Exception as e:
            self.logger.exception(e)
            user = None
        if not user:
            msg = 'Token {token} is invalid in request {rid}.'.format(
                token=header_token,
                rid=id(orig_request)
            )
            self.logger.error(msg)
            return JsonResponse(
                {
                    self.status_param: exceptions.auth_error.status,
                    self.msg_param: exceptions.auth_error.msg,
                    self.data_param: exceptions.auth_error.data
                },
                status=401
            )
        request.user = user
        msg = 'User {uid} login with token {token} in request {rid}.'.format(
            uid=user.id,
            token=header_token,
            rid=id(orig_request)
        )
        self.logger.info(msg)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/10/18
 """
from __future__ import unicode_literals, absolute_import
from djdg_core.common_api.public import common_client


def send_code(phone, content=None):
    """
    发送短信验证码到手机
    :param phone:
    :param content:
    :return:
    """
    res = common_client.get_resource('sms')
    data = {
        'phone': phone
    }
    if content:
        data['content'] = content
    ret = res.post.code(data=data)
    return ret.result


def send_voice(phone):
    """
    发送语音验证码
    :param phone:
    :return:
    """
    res = common_client.get_resource('sms')
    ret = res.post.voicecode(data={'phone': phone})
    return ret.result


def code_verify(phone, code):
    """
    验证码校验
    :param phone:
    :param code:
    :return:
    """
    data = {
        'phone': phone,
        'code': code
    }
    res = common_client.get_resource('code', 'sms')
    ret = res.post.verify(data=data)
    return ret.result



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/10/31
 """
from __future__ import unicode_literals, absolute_import
from djdg_core.common_api.public import common_client


def certificate_idcard(name, identity):
    """
    身份证实名认证
    :param name:
    :param identity:
    :return:
    """
    res = common_client.get_resource('certification')
    ret = res.post.idcard(data={'name': name, 'identity': identity})
    return ret.result


def certificate_bankcard(name, identity, bankcard, tel):
    """
    银行卡实名认证
    :param name:
    :param identity:
    :param bankcard:
    :param tel:
    :return:
    """
    res = common_client.get_resource('certification')
    data = {
        'name': name,
        'identity': identity,
        'bankcard': bankcard,
        'tel': tel
    }
    ret = res.post.bankcard(data=data)
    return ret.result


def bankcard_info(bankcard):
    """
    查询银行卡信息
    :param bankcard:
    :return:
    """
    res = common_client.get_resource('bankcard', app='certification')
    action = 'query/{}'.format(bankcard)
    ret = res.do_request('get', action)
    return ret.data



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/10/22
 """
from __future__ import unicode_literals, absolute_import
from rest_framework import viewsets
from rest_framework.response import Response


class NoneOptionsViewSet(viewsets.GenericViewSet):
    """
    options返回空的视图
    """
    def options(self, request, *args, **kwargs):
        return Response(data={})



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/10/18
 """
from __future__ import unicode_literals, absolute_import
from django.forms.models import model_to_dict
from django.contrib.auth import get_user_model
from django.utils.module_loading import import_string
from djdg_core import settings
from djdg_core.rest_client import RestClient
import logging
from djdg_core.cache.redis_client import cache
from djdg_core.common_api.user import get_info_by_token


user_sys_client = RestClient(
    host=settings.USER_SYSTEM_HOST,
    base_url=settings.USER_SYSTEM_BASE_URL,
    protocol=settings.USER_SYSTEM_PROTOCOL,
    logger=logging.getLogger(settings.LOGGER_NAME),
    oauth_app_id=settings.USER_SYSTEM_AUTH_APP_ID,
    oauth_app_secret=settings.USER_SYSTEM_AUTH_APP_SECRET
)


def user_fetcher(uid, token):
    """
    获取用户信息
    :param uid:
    :param token:
    :return:
    """
    res = user_sys_client.get_resource(settings.USER_SYSTEM_USER_RESOURCE, token=token)
    ret = res.get(action_to=uid)
    return ret.data


ext_user_cfg = settings.EXT_USER
if ext_user_cfg:
    ext_model = import_string(ext_user_cfg['MODEL'])
    ext_user_cfg['MODEL'] = ext_model


class UserProxy(object):
    """
    用户数据代理
    """
    def __init__(self, uid=None, token=None):
        self.token = token
        self.id = self.pk = uid
        self._data_ = None
        self._fetch_id()

    def _fetch_id(self):
        uid = self.id
        if not uid:
            uid = cache.get(self.token)
        if not uid:
            return None
        self.id = self.pk = int(uid)

    @property
    def data(self):
        if not self._data_:
            self._data_ = self._fetch_data()
        elif '__ext_data' not in self._data_:
            self._data_['__ext_data'] = self._fetch_ext_data()
        return self._data_

    def _fetch_data(self):
        return user_fetcher(self.id, self.token)

    def __getattr__(self, item):
        if item in self.data:
            return self.data[item]
        elif item in self.data['__ext_data']:
            return self.data['__ext_data'][item]
        raise AttributeError("'{}' object has no attribute '{}'".format(self.__class__.__name__, item))

    def _fetch_ext_data(self):
        if not ext_user_cfg:
            return {}
        ext_model = ext_user_cfg['MODEL']
        foreign_key = ext_user_cfg['FOREIGN_KEY']
        ext_user_name = ext_user_cfg.get('NAME', ext_model.__name__).lower()
        filter_fields = {
            foreign_key: self.id
        }
        ext_ist = ext_model.objects.get(**filter_fields)
        ext_data = model_to_dict(ext_ist)
        ext_data[ext_user_name] = ext_ist
        return ext_data


class ModelUserProxy(UserProxy):
    """
    数据模型方式的用户代理
    """
    def _fetch_data(self):
        user_model = get_user_model()
        user = user_model.objects.get(pk=self.id)
        return model_to_dict(user)


class CommonUserProxy(UserProxy):
    """
    公共模块用户代理
    """
    def _fetch_id(self):
        self._fetch_data()

    def _fetch_data(self):
        self._data_ = get_info_by_token(self.token)
        self.id = self.pk = self._data_['id']


try:
    _user_proxy_ = import_string(settings.USER_PROXY)
except:
    _user_proxy_ = UserProxy


def get_user_proxy():
    return _user_proxy_



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/12/19
 """
from __future__ import unicode_literals, absolute_import
from django.db import transaction
from django.forms.models import model_to_dict
from rest_framework import viewsets
from rest_framework import mixins
from rest_framework.decorators import list_route
from rest_framework.decorators import detail_route
from rest_framework.response import Response
from djdg_core.core_auth import serializers
from djdg_core.exceptions import DifferentPasswordError
from djdg_core.exceptions import CodeVerifyError
from djdg_core.common_api import sms
from djdg_core.exceptions import ParameterNotExistOrEmptyError
from djdg_core.exceptions import UserNameExistsError
from djdg_core.exceptions import UserNotExists
from djdg_core.exceptions import PasswordError
from djdg_core.exceptions import PermissionError
from djdg_core.exceptions import StanderResponseError
from djdg_core.core_auth.decorators import login_required
from djdg_core.core_auth.token import set_token
from djdg_core.core_auth import signals
from djdg_core.rest_permissions import allow_any


class UserViewSet(mixins.RetrieveModelMixin, viewsets.GenericViewSet):
    """
    用户视图
    """
    # 保存，更新，创建使用的 序列化对象
    serializer_class = serializers.UserSerializer
    # 修改密码使用的 序列化对象
    change_password_serializer_class = serializers.ChangePasswordUserSerializer
    # 返回给客户端使用的 序列化对象
    view_serializer_class = serializers.UserSerializer
    # 用户基础数据对象集合
    queryset = serializers.User.objects.all()
    # 返回给用户端用户数据集合
    view_queryset = serializers.User.objects.all()
    username_field = 'username'
    foreign_field = 'user'

    @transaction.atomic
    def create(self, request, *args, **kwargs):
        self.verify(request.data, *args, **kwargs)
        user_data = self.register(request.data, *args, **kwargs)
        instance = self.combine(user_data, *args, **kwargs)
        if 'token' not in user_data:
            token = self.create_token(user_data, *args, **kwargs)
            kwargs['token'] = token
        else:
            kwargs['token'] = user_data['token']
        signals.login_event.send(
            None,
            user=instance
        )
        kwargs['user_data'] = user_data
        return self.do_response(instance, *args, **kwargs)

    @list_route(methods=['post'])
    @allow_any
    @transaction.atomic
    def login(self, request, *args, **kwargs):
        """
        登录
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        login_type = self.check(request.data, *args, **kwargs)
        kwargs['login_type'] = login_type
        user_data = self.fetch_user(request.data, *args, **kwargs)
        instance = self.combine(user_data, *args, **kwargs)
        if 'token' not in user_data:
            token = self.create_token(user_data, *args, **kwargs)
            kwargs['token'] = token
        else:
            kwargs['token'] = user_data['token']
        signals.login_event.send(
            None,
            user=instance
        )
        kwargs['user_data'] = user_data
        return self.do_response(instance, *args, **kwargs)

    def create_token(self, user_data, *args, **kwargs):
        return set_token(user_data['id'])

    @login_required
    @transaction.atomic
    def update(self, request, *args, **kwargs):
        """
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        self.verify_permissions(request, *args, **kwargs)
        instance = self.get_object()
        instance = self.update_user(instance, request.data, partial=True)
        instance = self.update_ext_user(instance, request.data, partial=True)
        return self.do_response(instance)

    @detail_route(methods=['post'])
    @login_required
    @transaction.atomic
    def change_password(self, request, *args, **kwargs):
        """
        修改密码
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        self.verify_permissions(request, *args, **kwargs)
        instance = self.get_object()
        data = request.data
        data['id'] = instance.id
        chp_ser = self.get_change_password_serializer(data=data)
        chp_ser.is_valid(raise_exception=True)
        chp_ser.save()
        return self.do_response(instance, *args, **kwargs)

    def verify_permissions(self, request, *args, **kwargs):
        instance = self.get_object()
        if instance.id != request.user.id:
            raise PermissionError()

    def get_change_password_serializer(self, *args, **kwargs):
        """

        :param args:
        :param kwargs:
        :return:
        """
        klass = self.change_password_serializer_class
        kwargs['context'] = self.get_serializer_context()
        return klass(*args, **kwargs)

    def update_user(self, instance, user_data, *args, **kwargs):
        """
        更新基本用户信息
        :param instance:
        :param args:
        :param kwargs:
        :return:
        """
        partial = kwargs.pop('partial', False)
        serializer = self.get_serializer(instance=instance, data=user_data, partial=partial)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return serializer.instance

    def update_ext_user(self, instance, user_data, *args, **kwargs):
        """
        更新用户扩展信息
        :param instance:
        :param args:
        :param kwargs:
        :return: instance
        """
        return instance

    def check(self, login_data, *args, **kwargs):
        """
        检查登录时的参数，不正确则抛出异常
        :param login_data:
        :param args:
        :param kwargs:
        :return:
        """
        self.verify_fields(login_data, [self.username_field])
        if 'password' in login_data:
            filter_kwargs = {
                self.username_field: login_data[self.username_field]
            }
            user = self.get_queryset().filter(**filter_kwargs).first()
            if not user:
                raise UserNotExists(login_data[self.username_field])
            if not user.check_password(login_data['password']):
                raise PasswordError(login_data[self.username_field])
            return 'password'
        if 'code' in login_data:
            res = sms.code_verify(login_data[self.username_field], login_data['code'])
            if res.get('statusCode', None) != 0:
                ex = CodeVerifyError()
                msg = res.get('msg', None)
                if msg:
                    ex.message = msg
                raise ex
            return 'code'

    def fetch_user(self, login_data, *args, **kwargs):
        """
        拉取基础用户数据
        :param login_data:
        :param args:
        :param kwargs:
        :return: user data dict
        """
        login_type = kwargs.get('login_type')
        filter_kwargs = {
            self.username_field: login_data[self.username_field]
        }
        user = self.get_queryset().filter(**filter_kwargs).first()
        if not user and 'code' == login_type:
            # create user
            for f in ['first_name', 'last_name', 'email', 'password']:
                if f not in login_data:
                    login_data[f] = ''
            user_data = self.register(login_data, *args, **kwargs)
        else:
            user_data = model_to_dict(user)
        return user_data

    @classmethod
    def verify_fields(cls, data, fields):
        """
        验证字段是否存在
        :param data:
        :param fields:
        :return:
        """
        for f in fields:
            if f not in data:
                raise ParameterNotExistOrEmptyError(f)

    def verify(self, user_data, *args, **kwargs):
        """
        验证注册数据的正确性
        :param user_data:
        :param args:
        :param kwargs:
        :return: raise exception if verify failed
        """
        self.verify_fields(user_data, [self.username_field, 'password'])
        password = user_data['password']
        rep_password = user_data.pop('rep_password', None)
        code = user_data.pop('code', None)
        if password:
            if rep_password != password:
                raise DifferentPasswordError()
        elif code:
            res = sms.code_verify(user_data[self.username_field], code)
            if res.get('statusCode', None) != 0:
                ex = CodeVerifyError()
                msg = res.get('msg', None)
                if msg:
                    ex.message = msg
                raise ex
        else:
            raise StanderResponseError(6, '未知错误！')

    def register(self, user_data, *args, **kwargs):
        """
        将基础用户数据和扩展用户数据合并
        :param user_data:
        :param args:
        :param kwargs:
        :return: user data of dict
        """
        filter_kwargs = {
            self.username_field: user_data[self.username_field]
        }
        with transaction.atomic():
            user = self.get_queryset().filter(**filter_kwargs).select_for_update().first()
            if user:
                raise UserNameExistsError()
            serializer = self.get_serializer(data=user_data)
            serializer.is_valid(raise_exception=True)
            serializer.save()
            signals.register_event.send(
                None,
                user=serializer.instance
            )
            return model_to_dict(serializer.instance)

    def combine(self, user_data, *args, **kwargs):
        """
        合并用户扩展信息
        :param user_data:
        :param args:
        :param kwargs:
        :return: django model instance
        """
        filter_kwargs = {
            self.foreign_field: user_data['id']
        }
        instance = self.get_view_queryset().filter(**filter_kwargs).first()
        if not instance and 'code' == kwargs.get('login_type'):
            instance = self.create_ext_user(user_data, *args, **kwargs)
        if not instance:
            raise UserNotExists(user_data[self.username_field])
        return instance

    def create_ext_user(self, user_data, *args, **kwargs):
        return ''

    def get_view_queryset(self):
        return self.view_queryset.all()

    def do_response(self, instance, *args, **kwargs):
        """
        :param instance:
        :param args:
        :param kwargs:
        :return: response
        """
        serializer = self.get_view_serializer(instance=instance, user_data=kwargs.get('user_data'))
        data = serializer.data
        if 'token' in kwargs:
            data['token'] = kwargs['token']
        return Response(data=data)

    def get_view_serializer(self, *args, **kwargs):
        klass = self.view_serializer_class
        context = self.get_serializer_context()
        context['user_data'] = kwargs.pop('user_data', None)
        kwargs['context'] = context
        return klass(*args, **kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/9/18
 """
from __future__ import unicode_literals, absolute_import
from rest_framework import filters
from django_filters import Filter
from djdg_core.forms.fields import TimestampField


class TimestampFilter(Filter):
    """
    时间戳类型的filter
    """
    field_class = TimestampField


class SelectRelatedFilterBackend(filters.BaseFilterBackend):
    """
    增加select_related参数
    """
    def filter_queryset(self, request, queryset, view):
        select_related_fields = getattr(view, 'select_related_fields', None)
        if select_related_fields:
            queryset = queryset.select_related(*select_related_fields)
        return queryset



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/12/14
 """
from __future__ import unicode_literals, absolute_import
from djdg_core.cache.proxy import CacheProxy
from djdg_core import settings
from uuid import uuid1


token_settings = settings.cache_token_settings


class TokenCache(CacheProxy):
    """
    token在redis中的key
    """
    FORMAT = token_settings['FORMAT']
    KEY_EXPIRE = token_settings['EXPIRE']

    def __init__(self, token, engine=None):
        super(TokenCache, self).__init__(engine, token=token)
        self.settings = token_settings


def set_token(user_id):
    token = str(uuid1())
    token_cache = TokenCache(token=token)
    token_cache.hook_set(user_id, expire=True)
    return token


def get_user_id(token):
    """
    通过token获取user id
    :param token: uuid1 产生的内容
    :return:
    """
    token_cache = TokenCache(token=token)
    user_id = token_cache.hook_get()
    return int(user_id) if user_id else user_id



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/10/18
 """
from __future__ import unicode_literals, absolute_import
from djdg_core.common_api.public import common_client
from djdg_core.exceptions import ParameterNotExistOrEmptyError
from djdg_core.common_api.utils import regular_user


def register_by_code(code, phoneNumber, introducerCode='', unionid=''):
    """
    通过短信验证码注册
    :param code:
    :param phoneNumber:
    :param introducerCode:
    :param unionid:
    :return:
    """
    res = common_client.get_resource('regist')
    ret = res.post.code(data={
        'code': code,
        'phoneNumber': phoneNumber,
        'introducerCode': introducerCode,
        'unionid': unionid
    })
    return regular_user(ret.data)


def register_by_password(password, phoneNumber, introducerCode='', unionid=''):
    """
    通过密码方式注册
    :param password:
    :param phoneNumber:
    :param introducerCode:
    :param unionid:
    :return:
    """
    res = common_client.get_resource('regist')
    ret = res.post.password(data={
        'password': password,
        'phoneNumber': phoneNumber,
        'introducerCode': introducerCode,
        'unionid': unionid
    })
    return ret.result


def login_by_password(phoneNumber, password='', userkey=''):
    """
    密码方式登录
    :param phoneNumber:
    :param password:
    :param userkey:
    :return:
    """
    if not password and not userkey:
        raise ParameterNotExistOrEmptyError('password')
    res = common_client.get_resource('login')
    ret = res.post.password(data={
        'phoneNumber': phoneNumber,
        'password': password,
        'userkey': userkey
    })
    return regular_user(ret.data)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/7/12
 """
from __future__ import unicode_literals, absolute_import
from django.conf import settings
from redis import StrictRedis
import re


class MyStrictRedis(StrictRedis):
    """
    copy from o2o.cache to use REDIS.
    REDIS代理，将用到KEYS方法的KEY放入一个SET，使用KEYS操作时在此SET中查找
    """
    key_patterns = ('.*:column:map:set', '.*:groupon:set',
                    '.*:region:.*:promotion:set', 'seckill:.*',
                    '.*:seckill:set', 'smscode:.*', 'group:buying:*')

    def get_pattern_key(self, key):
        """
        """
        for pattern in self.key_patterns:
            if re.match(pattern, key):
                return pattern
        return None

    def keys(self, pattern='*'):
        pattern_key = self.get_pattern_key(pattern)
        if pattern_key is None:
            return ()

        result = []
        pattern = pattern.replace('*', '.*')
        keys = self.smembers(pattern_key)
        if keys is None:
            return ()
        for key in keys:
            if re.match(pattern, key):
                result.append(key)

        return result

    def add_key(self, key):
        pattern_key = self.get_pattern_key(key)
        if pattern_key is not None:
            super(MyStrictRedis, self).sadd(pattern_key, key)

    def remove_key(self, key):
        pattern_key = self.get_pattern_key(key)
        # print pattern_key,key
        if pattern_key is not None:
            self.srem(pattern_key, key)

    def brpoplpush(self, listname, back_list):
        darr = super(MyStrictRedis, self).brpop(listname)
        if darr is None:
            return None
        data = darr[1]
        super(MyStrictRedis, self).lpush(back_list, data)
        return data

    def execute_command(self, *args, **options):
        try:
            return super(MyStrictRedis, self).execute_command(*args, **options)
        except Exception as e:
            print e.message
            pass

    def clean_keys(self, pattern_key):
        if pattern_key in self.key_patterns:
            names = super(MyStrictRedis, self).smembers(pattern_key)
            if not names:
                names = set()
            names.add(pattern_key)
            super(MyStrictRedis, self).delete(*names)

    def delete(self, *names):
        for name in names:
            self.remove_key(name)
        return super(MyStrictRedis, self).delete(*names)

    def hset(self, name, key, value):
        self.add_key(name)
        return super(MyStrictRedis, self).hset(name, key, value)

    def set(self, name, value, ex=None, px=None, nx=False, xx=False):
        self.add_key(name)
        return super(MyStrictRedis, self).set(name, value, ex, px, nx, xx)

    def rpush(self, name, *values):
        self.add_key(name)
        return super(MyStrictRedis, self).rpush(name, *values)

    def lpush(self, name, *values):
        self.add_key(name)
        return super(MyStrictRedis, self).lpush(name, *values)

    def sadd(self, name, *values):
        # log.info(name)
        self.add_key(name)
        return super(MyStrictRedis, self).sadd(name, *values)


class DefaultCacheProxy(object):
    """
    Proxy access to the default Cache object's attributes.

    This allows the legacy `cache` object to be thread-safe using the new
    ``caches`` API.
    """
    def __init__(self):
        self._client = None

    @property
    def client(self):
        if not self._client:
            kwargs = {}
            for k, v in settings.REDIS.items():
                kwargs[k.lower()] = v
            self._client = MyStrictRedis(**kwargs)
        return self._client

    def __getattr__(self, name):
        return getattr(self.client, name)


cache = DefaultCacheProxy()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/12/26
 """
from __future__ import unicode_literals, absolute_import
from math import radians, cos, sin, asin, sqrt
import Geohash


# 经度1，纬度1，经度2，纬度2 （十进制度数）
def haversine(lon1, lat1, lon2, lat2, ndigits=2):
    """
    Calculate the great circle distance between two points
    on the earth (specified in decimal degrees)
    """
    # 将十进制度数转化为弧度
    lon1, lat1, lon2, lat2 = map(radians, [lon1, lat1, lon2, lat2])

    # haversine公式
    dlon = lon2 - lon1
    dlat = lat2 - lat1
    a = sin(dlat/2)**2 + cos(lat1) * cos(lat2) * sin(dlon/2)**2
    c = 2 * asin(sqrt(a))
    r = 6371 # 地球平均半径，单位为公里
    v = c * r * 1000
    return round(v, ndigits)


LONGITUDE_UNIT = 360.0 / 2 ** 15
LATITUDE_UNIT = 180.0 / 2 ** 15
LONGITUDE_LENGTH_UNIT = haversine(0, 0, LATITUDE_UNIT, 0)
LATITUDE_LENGTH_UNIT = haversine(0, 0, 0, LATITUDE_UNIT)
GEOHASH_LENGTH = 6


# 按照范围查找目标点相邻的区域
def range_geohash(lon, lat, dist_range=3000):
    """
    :param lon:
    :param lat:
    :param dist_range:
    :return:
    """
    lon_blocks = max(1, int(dist_range / LONGITUDE_LENGTH_UNIT))
    lat_blocks = max(1, int(dist_range / LATITUDE_LENGTH_UNIT))
    if lon_blocks % 2:
        lon_blocks += 1
    if lat_blocks % 2:
        lat_blocks += 1
    lon_blocks, lat_blocks = lon_blocks / 2, lat_blocks / 2
    geohash_set = set()
    for i in xrange(lat_blocks + 1):
        for j in xrange(lon_blocks):
            geohash_set.add(
                Geohash.encode(lat, lon + (j + 1) * LONGITUDE_UNIT, GEOHASH_LENGTH)
            )
            geohash_set.add(
                Geohash.encode(lat, lon - (j + 1) * LONGITUDE_UNIT, GEOHASH_LENGTH)
            )
        if 0 == i:
            geohash_set.add(
                Geohash.encode(lat, lon, GEOHASH_LENGTH)
            )
        else:
            geohash_set.add(
                Geohash.encode(lat + i * LATITUDE_UNIT, lon, GEOHASH_LENGTH)
            )
            geohash_set.add(
                Geohash.encode(lat - i * LATITUDE_UNIT, lon, GEOHASH_LENGTH)
            )
    return list(geohash_set)


GEOHASH_PRECISION = 12


# 对经纬度进行编码，统一使用12位精度
def geohash_encode(lon, lat):
    return Geohash.encode(lat, lon, GEOHASH_PRECISION)


def geohash_decode(code):
    return Geohash.decode(code)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/8/6
 """
from __future__ import unicode_literals, absolute_import
from rest_framework.authentication import BaseAuthentication


class SimpleRestAuthentication(BaseAuthentication):
    """
    rest 接口认证，在SessionAuthentication的基础上取出csrf的检查
    """
    def authenticate(self, request):
        user = getattr(request._request, 'user', None)
        if not user or not user.is_authenticated():
            return None
        return (user, None)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/11/7
 """
from __future__ import unicode_literals, absolute_import
from djdg_core.common_api.public import common_client


def file_upload(file_info):
    """
    上传文件
    :param file_info: (filename, file_obj[, mime_type])
    :return:
    """
    res = common_client.get_resource('upload', app='file')
    form_data = {
        'uploadedFile': file_info
    }
    ret = res.do_request(method='post', action=common_client.oauth_app_id, multi_part_fields=form_data, end_slash=False)
    return ret.data



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/10/18
 """
from setuptools import setup
import os


def get_packages(package):
    """
    Return root package and all sub-packages.
    """
    return [dirpath
            for dirpath, dirnames, filenames in os.walk(package)
            if os.path.exists(os.path.join(dirpath, '__init__.py'))]


def get_package_data(package):
    """
    Return all files under the root package, that are not in a
    package themselves.
    """
    walk = [(dirpath.replace(package + os.sep, '', 1), filenames)
            for dirpath, dirnames, filenames in os.walk(package)
            if not os.path.exists(os.path.join(dirpath, '__init__.py'))]

    filepaths = []
    for base, filenames in walk:
        filepaths.extend([os.path.join(base, filename)
                          for filename in filenames])
    return {package: filepaths}


version = '0.8.9'


setup(
    name='djdgcore',
    version=version,
    author='lex li',
    author_email='lexian.li@hey900.com',
    packages=get_packages('djdg_core'),
    package_data=get_package_data('djdg_core'),
    zip_safe=False
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/8/6
 """
from __future__ import unicode_literals, absolute_import
from djdg_core.core_auth.proxy import get_user_proxy


UserProxy = get_user_proxy()


class User(object):
    """
    用户信息封装
    """
    USER_TYPE = 1

    def __init__(self, uid=None, token=None):
        self.__proxy = UserProxy(uid, token)

    def is_authenticated(self):
        return self.id is not None

    def has_perms(self, perms):
        pass

    # @property
    # def perms(self):
    #     if self.__perms:
    #         return self.__perms
    #     res = user_sys_client.get_resource('perms').get(action_to=self.id)
    #     self.__perms = res.data
    #     return self.__perms

    @property
    def is_active(self):
        return True

    def __getattr__(self, item):
        return getattr(self.__proxy, item)


class OpenUser(object):
    """
    OAUTH user
    """
    USER_TYPE = 2

    def __init__(self, app_id, secret):
        self.app_id = app_id
        self.secret = secret

    def is_authenticated(self):
        return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/10/18
 """
from __future__ import unicode_literals, absolute_import
from djdg_core.common_api.public import common_client
from djdg_core.common_api.utils import regular_user


def get_info_by_username(username):
    """
    获取用户信息
    :param username:
    :return:
    """
    res = common_client.get_resource('user')
    ret = res.post.info(data={'username': username})
    return regular_user(ret.data)


def get_info_by_token(token):
    """
    通过token获取用户信息
    :param token:
    :return:
    """
    res = common_client.get_resource('user')
    ret = res.post.info(data={}, headers={'token': token})
    return regular_user(ret.data)


def user_info_modify(user_info):
    """
    修改用户信息
    :param user_info:
        {
            'username': '',
            'telephone': '',
            'birth': '',
            'sex': '',
            'email': '',
            'headimgurl': ''
        }
    :return:
    """
    res = common_client.get_resource('user')
    ret = res.post.modify(data=user_info)
    return ret.data



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/11/3
 """
from __future__ import unicode_literals, absolute_import
from djdg_core.rest_client import RestClient
from djdg_core import settings
import logging


logger = logging.getLogger(settings.LOGGER_NAME)


memberbao_client = RestClient(
    settings.MEMBERBAO_HOST,
    settings.MEMBERBAO_BASE_URL,
    settings.MEMBERBAO_PROTOCOL,
    oauth_app_id=settings.MEMBERBAO_AUTH_APP_ID,
    oauth_app_secret=settings.MEMBERBAO_AUTH_APP_SECRET,
    logger=logger
)
